@@ -10,7 +10,7 @@
  *
  *   The license which is distributed with this software in the file COPYRIGHT
  *   applies to this software. If your distribution is missing this file, you
- *   may request it from <pekkas@netcore.fi>.
+ *   may request it from <reubenhwk@gmail.com>.
  *
  */
 
@@ -19,8 +19,8 @@
 #include "radvd.h"
 #include "pathnames.h"
 
-int privsep_set(const char *iface, const char *var, uint32_t val);
-void privsep_read_loop(void);
+static int set_interface_var(const char *iface, const char *var, const char *name, uint32_t val);
+static void privsep_read_loop(void);
 
 /* For reading or writing, depending on process */
 static int pfd = -1;
@@ -41,37 +41,32 @@ struct privsep_command {
 };
 
 /* Privileged read loop */
-void
-privsep_read_loop(void)
+static void privsep_read_loop(void)
 {
-	struct privsep_command cmd;
-	int ret;
-
 	while (1) {
-		ret = readn(pfd, &cmd, sizeof(cmd));
+		struct privsep_command cmd;
+		int ret = readn(pfd, &cmd, sizeof(cmd));
 		if (ret <= 0) {
 			/* Error or EOF, give up */
 			if (ret < 0) {
-				flog(LOG_ERR, "Exiting, privsep_read_loop had readn error: %s\n",
-				     strerror(errno));
+				flog(LOG_ERR, "Exiting, privsep_read_loop had readn error: %s", strerror(errno));
 			} else {
-				flog(LOG_ERR, "Exiting, privsep_read_loop had readn return 0 bytes\n");
+				flog(LOG_ERR, "Exiting, privsep_read_loop had readn return 0 bytes");
 			}
-			close(pfd);
-			_exit(0);
 		}
 		if (ret != sizeof(cmd)) {
 			/* Short read, ignore */
 			return;
 		}
 
-		cmd.iface[IFNAMSIZ-1] = '\0';
+		cmd.iface[IFNAMSIZ - 1] = '\0';
 
-		switch(cmd.type) {
+		switch (cmd.type) {
 
 		case SET_INTERFACE_LINKMTU:
 			if (cmd.val < MIN_AdvLinkMTU || cmd.val > MAX_AdvLinkMTU) {
-				flog(LOG_ERR, "(privsep) %s: LinkMTU (%u) is not within the defined bounds, ignoring", cmd.iface, cmd.val);
+				flog(LOG_ERR, "(privsep) %s: LinkMTU (%u) is not within the defined bounds, ignoring",
+				     cmd.iface, cmd.val);
 				break;
 			}
 			ret = set_interface_var(cmd.iface, PROC_SYS_IP6_LINKMTU, "LinkMTU", cmd.val);
@@ -79,7 +74,8 @@ privsep_read_loop(void)
 
 		case SET_INTERFACE_CURHLIM:
 			if (cmd.val < MIN_AdvCurHopLimit || cmd.val > MAX_AdvCurHopLimit) {
-				flog(LOG_ERR, "(privsep) %s: CurHopLimit (%u) is not within the defined bounds, ignoring", cmd.iface, cmd.val);
+				flog(LOG_ERR, "(privsep) %s: CurHopLimit (%u) is not within the defined bounds, ignoring",
+				     cmd.iface, cmd.val);
 				break;
 			}
 			ret = set_interface_var(cmd.iface, PROC_SYS_IP6_CURHLIM, "CurHopLimit", cmd.val);
@@ -87,10 +83,13 @@ privsep_read_loop(void)
 
 		case SET_INTERFACE_REACHTIME:
 			if (cmd.val < MIN_AdvReachableTime || cmd.val > MAX_AdvReachableTime) {
-				flog(LOG_ERR, "(privsep) %s: BaseReachableTimer (%u) is not within the defined bounds, ignoring", cmd.iface, cmd.val);
+				flog(LOG_ERR,
+				     "(privsep) %s: BaseReachableTimer (%u) is not within the defined bounds, ignoring",
+				     cmd.iface, cmd.val);
 				break;
 			}
-			ret = set_interface_var(cmd.iface, PROC_SYS_IP6_BASEREACHTIME_MS, "BaseReachableTimer (ms)", cmd.val);
+			ret =
+			    set_interface_var(cmd.iface, PROC_SYS_IP6_BASEREACHTIME_MS, "BaseReachableTimer (ms)", cmd.val);
 			if (ret == 0)
 				break;
 			set_interface_var(cmd.iface, PROC_SYS_IP6_BASEREACHTIME, "BaseReachableTimer", cmd.val / 1000);
@@ -98,13 +97,14 @@ privsep_read_loop(void)
 
 		case SET_INTERFACE_RETRANSTIMER:
 			if (cmd.val < MIN_AdvRetransTimer || cmd.val > MAX_AdvRetransTimer) {
-				flog(LOG_ERR, "(privsep) %s: RetransTimer (%u) is not within the defined bounds, ignoring", cmd.iface, cmd.val);
+				flog(LOG_ERR, "(privsep) %s: RetransTimer (%u) is not within the defined bounds, ignoring",
+				     cmd.iface, cmd.val);
 				break;
 			}
 			ret = set_interface_var(cmd.iface, PROC_SYS_IP6_RETRANSTIMER_MS, "RetransTimer (ms)", cmd.val);
 			if (ret == 0)
 				break;
-			set_interface_var(cmd.iface, PROC_SYS_IP6_RETRANSTIMER, "RetransTimer", cmd.val / 1000 * USER_HZ); /* XXX user_hz */
+			set_interface_var(cmd.iface, PROC_SYS_IP6_RETRANSTIMER, "RetransTimer", cmd.val / 1000 * USER_HZ);	/* XXX user_hz */
 			break;
 
 		default:
@@ -115,32 +115,28 @@ privsep_read_loop(void)
 }
 
 /* Fork to create privileged process connected by a pipe */
-int
-privsep_init(void)
+int privsep_init(void)
 {
 	int pipefds[2];
-	pid_t pid;
 
 	if (pipe(pipefds) != 0) {
 		flog(LOG_ERR, "Couldn't create privsep pipe.");
-		return (-1);
+		return -1;
 	}
 
-	pid = fork();
+	pid_t pid = fork();
 	if (pid == -1) {
 		flog(LOG_ERR, "Couldn't fork for privsep.");
-		return (-1);
+		return -1;
 	}
 
 	if (pid == 0) {
-		int nullfd;
-
 		/* This will be the privileged child */
 		close(pipefds[1]);
 		pfd = pipefds[0];
 
 		/* Detach from stdio */
-		nullfd = open("/dev/null", O_RDONLY);
+		int nullfd = open("/dev/null", O_RDONLY);
 		if (nullfd < 0) {
 			perror("/dev/null");
 			close(pfd);
@@ -154,10 +150,12 @@ privsep_init(void)
 
 		privsep_read_loop();
 		close(pfd);
-		flog(LOG_ERR, "Exiting, privsep_read_loop is complete.\n");
+		flog(LOG_ERR, "Exiting, privsep_read_loop is complete.");
 		_exit(0);
 	}
 
+	dlog(LOG_DEBUG, 3, "radvd privsep PID is %d", pid);
+
 	/* Continue execution (will drop privileges soon) */
 	close(pipefds[0]);
 	pfd = pipefds[1];
@@ -166,8 +164,7 @@ privsep_init(void)
 }
 
 /* Interface calls for the unprivileged process */
-int
-privsep_interface_linkmtu(const char *iface, uint32_t mtu)
+int privsep_interface_linkmtu(const char *iface, uint32_t mtu)
 {
 	struct privsep_command cmd;
 	cmd.type = SET_INTERFACE_LINKMTU;
@@ -175,42 +172,76 @@ privsep_interface_linkmtu(const char *iface, uint32_t mtu)
 	cmd.val = mtu;
 
 	if (writen(pfd, &cmd, sizeof(cmd)) != sizeof(cmd))
-		return (-1);
+		return -1;
 	return 0;
 }
 
-int
-privsep_interface_curhlim(const char *iface, uint32_t hlim)
+int privsep_interface_curhlim(const char *iface, uint32_t hlim)
 {
 	struct privsep_command cmd;
 	cmd.type = SET_INTERFACE_CURHLIM;
 	strncpy(cmd.iface, iface, sizeof(cmd.iface));
 	cmd.val = hlim;
 	if (writen(pfd, &cmd, sizeof(cmd)) != sizeof(cmd))
-		return (-1);
+		return -1;
 	return 0;
 }
 
-int
-privsep_interface_reachtime(const char *iface, uint32_t rtime)
+int privsep_interface_reachtime(const char *iface, uint32_t rtime)
 {
 	struct privsep_command cmd;
 	cmd.type = SET_INTERFACE_REACHTIME;
 	strncpy(cmd.iface, iface, sizeof(cmd.iface));
 	cmd.val = rtime;
 	if (writen(pfd, &cmd, sizeof(cmd)) != sizeof(cmd))
-		return (-1);
+		return -1;
 	return 0;
 }
 
-int
-privsep_interface_retranstimer(const char *iface, uint32_t rettimer)
+int privsep_interface_retranstimer(const char *iface, uint32_t rettimer)
 {
 	struct privsep_command cmd;
 	cmd.type = SET_INTERFACE_RETRANSTIMER;
 	strncpy(cmd.iface, iface, sizeof(cmd.iface));
 	cmd.val = rettimer;
 	if (writen(pfd, &cmd, sizeof(cmd)) != sizeof(cmd))
-		return (-1);
+		return -1;
 	return 0;
 }
+
+/* note: also called from the root context */
+static int set_interface_var(const char *iface, const char *var, const char *name, uint32_t val)
+{
+	int retval = -1;
+	FILE * fp = 0;
+	char * spath = strdupf(var, iface);
+
+	/* No path traversal */
+	if (!iface[0] || !strcmp(iface, ".") || !strcmp(iface, "..") || strchr(iface, '/'))
+		goto cleanup;
+
+	if (access(spath, F_OK) != 0)
+		goto cleanup;
+
+	fp = fopen(spath, "w");
+	if (!fp) {
+		if (name)
+			flog(LOG_ERR, "failed to set %s (%u) for %s: %s", name, val, iface, strerror(errno));
+		goto cleanup;
+	}
+
+	if (0 > fprintf(fp, "%u", val)) {
+		goto cleanup;
+	}
+
+	retval = 0;
+
+cleanup:
+	if (fp)
+		fclose(fp);
+
+	free(spath);
+
+	return retval;
+}
+
