@@ -13,8 +13,7 @@
  *
  */
 
-#ifndef RADV_H
-#define RADV_H
+#pragma once
 
 #include "config.h"
 #include "includes.h"
@@ -35,61 +34,88 @@ struct Clients;
 #define HWADDR_MAX 16
 #define USER_HZ 100
 
+struct safe_buffer {
+	int should_free;
+	size_t allocated;
+	size_t used;
+	unsigned char * buffer;
+};
+
+#define SAFE_BUFFER_INIT (struct safe_buffer){.should_free = 0, .allocated = 0, .used = 0, .buffer = 0}
+
+
 struct Interface {
-	char Name[IFNAMSIZ];	/* interface name */
-
-	struct in6_addr if_addr;
-	unsigned int if_index;
-
-	uint8_t init_racount;	/* Initial RAs */
-
-	uint8_t if_hwaddr[HWADDR_MAX];
-	int if_hwaddr_len;
-	int if_prefix_len;
-	int if_maxmtu;
-
-	int cease_adv;
-
-	struct timeval last_ra_time;
+	struct Interface *next;
 
 	int IgnoreIfMissing;
 	int AdvSendAdvert;
 	double MaxRtrAdvInterval;
 	double MinRtrAdvInterval;
 	double MinDelayBetweenRAs;
-	int AdvManagedFlag;
-	int AdvOtherConfigFlag;
-	uint32_t AdvLinkMTU;
-	uint32_t AdvReachableTime;
-	uint32_t AdvRetransTimer;
-	uint8_t AdvCurHopLimit;
-	int32_t AdvDefaultLifetime;	/* XXX: really uint16_t but we need to use -1 */
-	int AdvDefaultPreference;
 	int AdvSourceLLAddress;
 	int UnicastOnly;
+	struct Clients *ClientList;
 
-	/* Mobile IPv6 extensions */
-	int AdvIntervalOpt;
-	int AdvHomeAgentInfo;
-	int AdvHomeAgentFlag;
-	uint16_t HomeAgentPreference;
-	int32_t HomeAgentLifetime;	/* XXX: really uint16_t but we need to use -1 */
+	struct state_info {
+		int ready;	/* Info whether this interface has been initialized successfully */
+		int changed;	/* Info whether this interface's settings have changed */
+		int cease_adv;
+		uint32_t racount;
+	} state_info;
 
-	/* NEMO extensions */
-	int AdvMobRtrSupportFlag;
+	struct properties {
+		char name[IFNAMSIZ];	/* interface name */
+		struct in6_addr if_addr;
+		unsigned int if_index;
+	} props;
+
+	struct ra_header_info {
+		int AdvManagedFlag;
+		int AdvOtherConfigFlag;
+		uint8_t AdvCurHopLimit;
+		int AdvHomeAgentFlag;
+		int32_t AdvDefaultLifetime;	/* XXX: really uint16_t but we need to use -1 */
+		int AdvDefaultPreference;
+		uint32_t AdvReachableTime;
+		uint32_t AdvRetransTimer;
+	} ra_header_info;
+
+	struct times {
+		struct timespec last_multicast;
+		struct timespec next_multicast;
+		struct timespec last_ra_time;
+	} times;
 
 	struct AdvPrefix *AdvPrefixList;
 	struct AdvRoute *AdvRouteList;
 	struct AdvRDNSS *AdvRDNSSList;
 	struct AdvDNSSL *AdvDNSSLList;
-	struct Clients *ClientList;
-	struct timeval last_multicast;
-	struct timeval next_multicast;
 
-	/* Info whether this interface has failed in the past (and may need to be reinitialized) */
-	int HasFailed;
+	uint32_t AdvLinkMTU; /* XXX: sllao also has an if_maxmtu value...Why? */
 
-	struct Interface *next;
+	struct sllao {
+		uint8_t if_hwaddr[HWADDR_MAX];
+		int if_hwaddr_len;
+		int if_prefix_len;
+		int if_maxmtu;
+	} sllao;
+
+	struct mipv6 {
+		/* Mobile IPv6 extensions */
+		int AdvIntervalOpt;
+		int AdvHomeAgentInfo;
+
+		uint16_t HomeAgentPreference;
+		int32_t HomeAgentLifetime;	/* XXX: really uint16_t but we need to use -1 */
+
+		/* NEMO extensions */
+		int AdvMobRtrSupportFlag;
+	} mipv6;
+
+	struct AdvLowpanCo *AdvLowpanCoList;
+	struct AdvAbro *AdvAbroList;
+
+	int lineno; /* On what line in the config file was this iface defined? */
 };
 
 struct Clients {
@@ -161,6 +187,26 @@ struct AdvDNSSL {
 	struct AdvDNSSL *next;
 };
 
+/* Options for 6lopan configuration */
+
+struct AdvLowpanCo {
+	uint8_t ContextLength;
+	uint8_t ContextCompressionFlag;
+	uint8_t AdvContextID;
+	uint16_t AdvLifeTime;
+	struct in6_addr AdvContextPrefix;
+
+	struct AdvLowpanCo *next;
+};
+
+struct AdvAbro {
+	uint16_t Version[2];
+	uint16_t ValidLifeTime;
+	struct in6_addr LBRaddress;
+
+	struct AdvAbro *next;
+};
+
 /* Mobile IPv6 extensions */
 
 struct AdvInterval {
@@ -178,35 +224,57 @@ struct HomeAgentInfo {
 	uint16_t lifetime;
 };
 
-/* gram.y */
-int yyparse(void);
+/* Uclibc : include/netinet/icmpv6.h - Added by Bhadram*/
+#define ND_OPT_ARO	33
+#define ND_OPT_6CO	34
+#define ND_OPT_ABRO	35
 
-/* scanner.l */
-int yylex(void);
+struct nd_opt_abro {
+	uint8_t nd_opt_abro_type;
+	uint8_t nd_opt_abro_len;
+	uint16_t nd_opt_abro_ver_low;
+	uint16_t nd_opt_abro_ver_high;
+	uint16_t nd_opt_abro_valid_lifetime;
+	struct in6_addr nd_opt_abro_6lbr_address;
+};
+
+struct nd_opt_6co {
+	uint8_t nd_opt_6co_type;
+	uint8_t nd_opt_6co_len;
+	uint8_t nd_opt_6co_context_len;
+	uint8_t nd_opt_6co_res:3;
+	uint8_t nd_opt_6co_c:1;
+	uint8_t nd_opt_6co_cid:4;
+	uint16_t nd_opt_6co_reserved;
+	uint16_t nd_opt_6co_valid_lifetime;
+	struct in6_addr nd_opt_6co_con_prefix;
+};				/*Added by Bhadram */
+
+/* gram.y */
+struct Interface *readin_config(char const *fname);
 
 /* radvd.c */
-int check_ip6_forwarding(void);
-void reload_config(void);
-void reset_prefix_lifetimes(void);
+int disable_ipv6_autoconfig(char const *iface);
 
 /* timer.c */
-struct timeval next_timeval(double next);
-int timevaldiff(struct timeval const *a, struct timeval const *b);
-int next_time_msec(struct Interface const *iface);
+struct timespec next_timespec(double next);
+int64_t timespecdiff(struct timespec const *a, struct timespec const *b);
+uint64_t next_time_msec(struct Interface const *iface);
 int expired(struct Interface const *iface);
 
 /* device.c */
-int update_device_info(struct Interface *);
-int check_device(struct Interface *);
+int update_device_index(struct Interface *iface);
+int update_device_info(int sock, struct Interface *);
+int check_device(int sock, struct Interface *);
 int setup_linklocal_addr(struct Interface *);
-int setup_allrouters_membership(struct Interface *);
-int check_allrouters_membership(struct Interface *);
+int setup_allrouters_membership(int sock, struct Interface *);
+int setup_linklocal_addr(struct Interface *iface);
 int get_v4addr(const char *, unsigned int *);
-int set_interface_var(const char *, const char *, const char *, uint32_t);
 int set_interface_linkmtu(const char *, uint32_t);
 int set_interface_curhlim(const char *, uint8_t);
 int set_interface_reachtime(const char *, uint32_t);
 int set_interface_retranstimer(const char *, uint32_t);
+int check_ip6_forwarding(void);
 
 /* interface.c */
 void iface_init_defaults(struct Interface *);
@@ -215,28 +283,38 @@ void route_init_defaults(struct AdvRoute *, struct Interface *);
 void rdnss_init_defaults(struct AdvRDNSS *, struct Interface *);
 void dnssl_init_defaults(struct AdvDNSSL *, struct Interface *);
 int check_iface(struct Interface *);
+void touch_iface(struct Interface * iface);
+int setup_iface(int sock, struct Interface *iface);
+void free_ifaces(struct Interface *ifaces);
+struct Interface *find_iface_by_index(struct Interface *iface, int index);
+struct Interface *find_iface_by_time(struct Interface *iface_list);
+void for_each_iface(struct Interface *ifaces, void (*foo) (struct Interface * iface, void *), void *data);
+void reschedule_iface(struct Interface *iface, double next);
 
 /* socket.c */
 int open_icmpv6_socket(void);
 
 /* send.c */
-int send_ra(struct Interface *iface, struct in6_addr *dest);
-int send_ra_forall(struct Interface *iface, struct in6_addr *dest);
-int really_send(struct in6_addr const *dest, unsigned int if_index, struct in6_addr if_addr, unsigned char *buff, size_t len);
+int send_ra_forall(int sock, struct Interface *iface, struct in6_addr *dest);
 
 /* process.c */
-void process(struct Interface *, unsigned char *, int, struct sockaddr_in6 *, struct in6_pktinfo *, int);
+void process(int sock, struct Interface *, unsigned char *, int, struct sockaddr_in6 *, struct in6_pktinfo *, int);
 
 /* recv.c */
-int recv_rs_ra(unsigned char *, struct sockaddr_in6 *, struct in6_pktinfo **, int *);
+int recv_rs_ra(int sock, unsigned char *, struct sockaddr_in6 *, struct in6_pktinfo **, int *);
 
 /* util.c */
+char * strdupf(char const * format, ...) __attribute__ ((format(printf, 1, 2)));
 double rand_between(double, double);
-void print_addr(struct in6_addr *, char *);
+void addrtostr(struct in6_addr *, char *, size_t);
 int check_rdnss_presence(struct AdvRDNSS *, struct in6_addr *);
 int check_dnssl_presence(struct AdvDNSSL *, const char *);
 ssize_t readn(int fd, void *buf, size_t count);
 ssize_t writen(int fd, const void *buf, size_t count);
+struct safe_buffer * new_safe_buffer(void);
+void safe_buffer_free(struct safe_buffer * sb);
+size_t safe_buffer_pad(struct safe_buffer * sb, size_t count);
+size_t safe_buffer_append(struct safe_buffer * sb, void const * m, size_t count);
 
 /* privsep.c */
 int privsep_init(void);
@@ -275,5 +353,3 @@ int privsep_interface_retranstimer(const char *iface, uint32_t rettimer);
 #endif
 #endif
 #endif
-
-#endif
