@@ -17,15 +17,10 @@
 #include "includes.h"
 #include "radvd.h"
 
-int recv_rs_ra(unsigned char *msg, struct sockaddr_in6 *addr, struct in6_pktinfo **pkt_info, int *hoplimit)
+int recv_rs_ra(int sock, unsigned char *msg, struct sockaddr_in6 *addr, struct in6_pktinfo **pkt_info, int *hoplimit)
 {
-	struct msghdr mhdr;
-	struct cmsghdr *cmsg;
-	struct iovec iov;
 	static unsigned char *chdr = NULL;
 	static unsigned int chdrlen = 0;
-	int len;
-	fd_set rfds;
 
 	if (!chdr) {
 		chdrlen = CMSG_SPACE(sizeof(struct in6_pktinfo)) + CMSG_SPACE(sizeof(int));
@@ -35,19 +30,11 @@ int recv_rs_ra(unsigned char *msg, struct sockaddr_in6 *addr, struct in6_pktinfo
 		}
 	}
 
-	FD_ZERO(&rfds);
-	FD_SET(sock, &rfds);
-
-	if (select(sock + 1, &rfds, NULL, NULL, NULL) < 0) {
-		if (errno != EINTR)
-			flog(LOG_ERR, "select: %s", strerror(errno));
-
-		return -1;
-	}
-
+	struct iovec iov;
 	iov.iov_len = MSG_SIZE_RECV;
 	iov.iov_base = (caddr_t) msg;
 
+	struct msghdr mhdr;
 	memset(&mhdr, 0, sizeof(mhdr));
 	mhdr.msg_name = (caddr_t) addr;
 	mhdr.msg_namelen = sizeof(*addr);
@@ -56,7 +43,7 @@ int recv_rs_ra(unsigned char *msg, struct sockaddr_in6 *addr, struct in6_pktinfo
 	mhdr.msg_control = (void *)chdr;
 	mhdr.msg_controllen = chdrlen;
 
-	len = recvmsg(sock, &mhdr, 0);
+	int len = recvmsg(sock, &mhdr, 0);
 
 	if (len < 0) {
 		if (errno != EINTR)
@@ -67,28 +54,31 @@ int recv_rs_ra(unsigned char *msg, struct sockaddr_in6 *addr, struct in6_pktinfo
 
 	*hoplimit = 255;
 
-	for (cmsg = CMSG_FIRSTHDR(&mhdr); cmsg != NULL; cmsg = CMSG_NXTHDR(&mhdr, cmsg)) {
+	for (struct cmsghdr *cmsg = CMSG_FIRSTHDR(&mhdr); cmsg != NULL; cmsg = CMSG_NXTHDR(&mhdr, cmsg)) {
 		if (cmsg->cmsg_level != IPPROTO_IPV6)
 			continue;
 
 		switch (cmsg->cmsg_type) {
 #ifdef IPV6_HOPLIMIT
 		case IPV6_HOPLIMIT:
-			if ((cmsg->cmsg_len == CMSG_LEN(sizeof(int))) && (*(int *)CMSG_DATA(cmsg) >= 0) && (*(int *)CMSG_DATA(cmsg) < 256)) {
+			if ((cmsg->cmsg_len == CMSG_LEN(sizeof(int))) && (*(int *)CMSG_DATA(cmsg) >= 0)
+			    && (*(int *)CMSG_DATA(cmsg) < 256)) {
 				*hoplimit = *(int *)CMSG_DATA(cmsg);
 			} else {
-				flog(LOG_ERR, "received a bogus IPV6_HOPLIMIT from the kernel! len=%d, data=%d", cmsg->cmsg_len, *(int *)CMSG_DATA(cmsg));
-				return (-1);
+				flog(LOG_ERR, "received a bogus IPV6_HOPLIMIT from the kernel! len=%zu, data=%d",
+				     cmsg->cmsg_len, *(int *)CMSG_DATA(cmsg));
+				return -1;
 			}
 			break;
 #endif				/* IPV6_HOPLIMIT */
 		case IPV6_PKTINFO:
-			if ((cmsg->cmsg_len == CMSG_LEN(sizeof(struct in6_pktinfo))) && ((struct in6_pktinfo *)CMSG_DATA(cmsg))->ipi6_ifindex) {
+			if ((cmsg->cmsg_len == CMSG_LEN(sizeof(struct in6_pktinfo)))
+			    && ((struct in6_pktinfo *)CMSG_DATA(cmsg))->ipi6_ifindex) {
 				*pkt_info = (struct in6_pktinfo *)CMSG_DATA(cmsg);
 			} else {
-				flog(LOG_ERR, "received a bogus IPV6_PKTINFO from the kernel! len=%d, index=%d",
+				flog(LOG_ERR, "received a bogus IPV6_PKTINFO from the kernel! len=%zu, index=%d",
 				     cmsg->cmsg_len, ((struct in6_pktinfo *)CMSG_DATA(cmsg))->ipi6_ifindex);
-				return (-1);
+				return -1;
 			}
 			break;
 		}
